'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

function isNullOrUndef(o) {
    return o === undefined || o === null;
}
function isInvalid(o) {
    return isNullOrUndef(o) || o === true || o === false;
}
function isVNode(node) {
    return !isNullOrUndef(node) && node.vtype === 2 /* Node */;
}
function isVText(node) {
    return !isNullOrUndef(node) && node.vtype === 1 /* Text */;
}
function isComposite(node) {
    return !isNullOrUndef(node) && node.vtype === 4 /* Composite */;
}
function isStateless(node) {
    return !isNullOrUndef(node) && node.vtype === 8 /* Stateless */;
}
// typescript will compile the enum's value for us.
// eg.
// Composite = 1 << 2  => Composite = 4
var VType;
(function (VType) {
    VType[VType["Text"] = 1] = "Text";
    VType[VType["Node"] = 2] = "Node";
    VType[VType["Composite"] = 4] = "Composite";
    VType[VType["Stateless"] = 8] = "Stateless";
    VType[VType["Void"] = 16] = "Void";
})(VType || (VType = {}));

// tslint:disable-next-line
var global = (function () {
    let local;
    if (typeof global !== 'undefined') {
        local = global;
    }
    else if (typeof self !== 'undefined') {
        local = self;
    }
    else {
        try {
            local = Function('return this')();
        }
        catch (e) {
            throw new Error('global object is unavailable in this environment');
        }
    }
    return local;
})();
const isBrowser = typeof window !== 'undefined';
// tslint:disable-next-line:no-empty
function noop$1() { }
const fakeDoc = {
    createElement: noop$1,
    createElementNS: noop$1,
    createTextNode: noop$1
};
const doc = isBrowser ? document : fakeDoc;

/* istanbul ignore next */
// tslint:disable-next-line
Object.is = Object.is || function (x, y) {
    if (x === y) {
        return x !== 0 || 1 / x === 1 / y;
    }
    return x !== x && y !== y;
};
function isNumber(arg) {
    return typeof arg === 'number';
}
const isSupportSVG = isFunction(doc.createAttributeNS);
function isString(arg) {
    return typeof arg === 'string';
}
function isFunction(arg) {
    return typeof arg === 'function';
}
const isArray = Array.isArray;
function extend(source, from) {
    if (!from) {
        return source;
    }
    for (const key in from) {
        if (from.hasOwnProperty(key)) {
            source[key] = from[key];
        }
    }
    return source;
}
function clone(obj) {
    return extend({}, obj);
}

const uppercasePattern = /[A-Z]/g;
const CssPropCache = {};
function getCssPropertyName(str) {
    if (CssPropCache.hasOwnProperty(str)) {
        return CssPropCache[str];
    }
    return (CssPropCache[str] =
        str.replace(uppercasePattern, '-$&').toLowerCase() + ':');
}
const isVoidElements = {
    'area': true,
    'base': true,
    'br': true,
    'col': true,
    'command': true,
    'embed': true,
    'hr': true,
    'img': true,
    'input': true,
    'keygen': true,
    'link': true,
    'meta': true,
    'param': true,
    'source': true,
    'track': true,
    'wbr': true
};
/**
 * CSS properties which accept numbers but are not in units of "px".
 */
const isUnitlessNumber = {
    animationIterationCount: true,
    borderImageOutset: true,
    borderImageSlice: true,
    borderImageWidth: true,
    boxFlex: true,
    boxFlexGroup: true,
    boxOrdinalGroup: true,
    columnCount: true,
    flex: true,
    flexGrow: true,
    flexPositive: true,
    flexShrink: true,
    flexNegative: true,
    flexOrder: true,
    gridRow: true,
    gridColumn: true,
    fontWeight: true,
    lineClamp: true,
    lineHeight: true,
    opacity: true,
    order: true,
    orphans: true,
    tabSize: true,
    widows: true,
    zIndex: true,
    zoom: true,
    // SVG-related properties
    fillOpacity: true,
    floodOpacity: true,
    stopOpacity: true,
    strokeDasharray: true,
    strokeDashoffset: true,
    strokeMiterlimit: true,
    strokeOpacity: true,
    strokeWidth: true
};
const entities = {
    '<': '&lt;',
    '>': '&gt;',
    '&': '&amp;',
    '"': '&quot;',
    '\\': '&#039;'
};
function encodeEntities(text) {
    if (typeof text !== 'string') {
        text = String(text);
    }
    return text.replace(/[<>\"\&\\]/g, (m) => entities[m]);
}

// tslint:disable-next-line:max-line-length
const skipAttributes = {
    ref: true,
    key: true,
    children: true,
    owner: true
};
function hashToClassName(obj) {
    const arr = [];
    for (const i in obj) {
        if (obj[i]) {
            arr.push(i);
        }
    }
    return arr.join(' ');
}
function renderStylesToString(styles) {
    if (isString(styles)) {
        return styles;
    }
    else {
        let renderedString = '';
        for (const styleName in styles) {
            const value = styles[styleName];
            if (isString(value)) {
                renderedString += `${getCssPropertyName(styleName)}${value};`;
            }
            else if (isNumber(value)) {
                renderedString += `${getCssPropertyName(styleName)}${value}${isUnitlessNumber[styleName] ? '' : 'px'};`;
            }
        }
        return renderedString;
    }
}
function renderVNodeToString(vnode, parent, context, isSvg) {
    if (isInvalid(vnode)) {
        return '';
    }
    const { type, props, children } = vnode;
    if (isVText(vnode)) {
        return encodeEntities(vnode.text);
    }
    else if (isVNode(vnode)) {
        let renderedString = `<${type}`;
        let html;
        if (!isNullOrUndef(props)) {
            for (let prop in props) {
                const value = props[prop];
                if (skipAttributes[prop]) {
                    continue;
                }
                if (prop === 'dangerouslySetInnerHTML') {
                    html = value.__html;
                }
                else if (prop === 'style') {
                    const styleStr = renderStylesToString(value);
                    renderedString += styleStr ? ` style="${renderStylesToString(value)}"` : '';
                }
                else if (prop === 'class' || prop === 'className') {
                    renderedString += ` class="${isString(value)
                        ? value
                        : hashToClassName(value)}"`;
                }
                else if (prop === 'defaultValue') {
                    if (!props.value) {
                        renderedString += ` value="${encodeEntities(value)}"`;
                    }
                }
                else if (prop === 'defaultChecked') {
                    if (!props.checked) {
                        renderedString += ` checked="${value}"`;
                    }
                }
                else if (isSvg && prop.match(/^xlink\:?(.+)/)) {
                    prop = prop.toLowerCase().replace(/^xlink\:?(.+)/, 'xlink:$1');
                    renderedString += ` ${prop}="${encodeEntities(value)}"`;
                }
                else {
                    if (isString(value)) {
                        renderedString += ` ${prop}="${encodeEntities(value)}"`;
                    }
                    else if (isNumber(value)) {
                        renderedString += ` ${prop}="${value}"`;
                    }
                    else if (value === true) {
                        renderedString += ` ${prop}`;
                    }
                }
            }
        }
        if (isVoidElements[type]) {
            renderedString += `/>`;
        }
        else {
            renderedString += `>`;
            if (html) {
                renderedString += html;
            }
            else if (!isInvalid(children)) {
                if (isString(children)) {
                    renderedString += children === '' ? ' ' : encodeEntities(children);
                }
                else if (isNumber(children)) {
                    renderedString += children + '';
                }
                else if (isArray(children)) {
                    for (let i = 0, len = children.length; i < len; i++) {
                        const child = children[i];
                        if (isString(child)) {
                            renderedString += child === '' ? ' ' : encodeEntities(child);
                        }
                        else if (isNumber(child)) {
                            renderedString += child;
                        }
                        else if (!isInvalid(child)) {
                            isSvg = type === 'svg' ? true : type === 'foreignObject' ? false : isSvg;
                            renderedString += renderVNodeToString(child, vnode, context, isSvg);
                        }
                    }
                }
                else {
                    isSvg = type === 'svg' ? true : type === 'foreignObject' ? false : isSvg;
                    renderedString += renderVNodeToString(children, vnode, context, isSvg);
                }
            }
            if (!isVoidElements[type]) {
                renderedString += `</${type}>`;
            }
        }
        return renderedString;
    }
    else if (isComposite(vnode)) {
        const instance = new type(props, context);
        instance._disable = true;
        instance.props = props;
        instance.context = context;
        if (isFunction(instance.componentWillMount)) {
            instance.componentWillMount();
        }
        const rendered = instance.render();
        if (isFunction(instance.getChildContext)) {
            context = extend(clone(context), instance.getChildContext());
        }
        return renderVNodeToString(rendered, vnode, context, isSvg);
    }
    else if (isStateless(vnode)) {
        const rendered = type(props, context);
        return renderVNodeToString(rendered, vnode, context, isSvg);
    }
}
function renderToString(input) {
    return renderVNodeToString(input, {}, {});
}
function renderToStaticMarkup(input) {
    return renderVNodeToString(input, {}, {});
}

exports.renderToString = renderToString;
exports.renderToStaticMarkup = renderToStaticMarkup;
//# sourceMappingURL=index.js.map
